@@ -18,7 +18,10 @@
 #   Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 #   MA 02139, USA.
 
-lrr.default <- function(x, eps = sqrt(.Machine$double.eps), ..., orthonormalize = TRUE) {
+lrr.default <- function(x, eps = sqrt(.Machine$double.eps),
+                        reverse = FALSE,
+                        ...,
+                        orthonormalize = TRUE) {
   if (orthonormalize) {
     U <- qr.Q(qr(x))
   } else {
@@ -30,16 +33,22 @@ lrr.default <- function(x, eps = sqrt(.Machine$double.eps), ..., orthonormalize
   # Return zero LRR coefficients for zero subspace
   if (ncol(U) == 0) return(rep(0, N - 1))
 
-  lpf <- Conj(U) %*% t(U[N, , drop = FALSE])
+  idx <- if (!reverse) N else 1
+  lpf <- Conj(U) %*% t(U[idx, , drop = FALSE])
 
-  divider <- 1 - lpf[N]
+  divider <- 1 - lpf[idx]
   if (Mod(divider) < eps)
     stop("Verticality coefficient equals to 1")
 
-  lpf[-N] / divider
+  lpf[-idx] / divider
 }
 
-lrr.1d.ssa <- function(x, groups, ..., drop = TRUE) {
+lrr.1d.ssa <- function(x, groups,
+                       reverse = FALSE,
+                       ..., drop = TRUE) {
+  if (is.shaped(x))
+    stop("`LRR is not implemented for shaped SSA case yet")
+
   if (missing(groups))
     groups <- 1:min(nsigma(x), nu(x))
 
@@ -48,7 +57,8 @@ lrr.1d.ssa <- function(x, groups, ..., drop = TRUE) {
 
   out <- list()
   for (i in seq_along(groups)) {
-    res <- lrr.default(.colspan(x, groups[[i]]), ..., orthonormalize = FALSE)
+      res <- lrr.default(.colspan(x, groups[[i]]), reverse = reverse,
+                         ..., orthonormalize = FALSE)
     class(res) <- "lrr"
 
     out[[i]] <- res
@@ -82,7 +92,8 @@ roots.lrr <- function(x, ..., method = c("companion", "polyroot")) {
   res[order(abs(res), decreasing = TRUE)]
 }
 
-apply.lrr <- function(F, lrr, len = 1, only.new = FALSE, drift = 0) {
+apply.lrr <- function(F, lrr, len = 1, only.new = FALSE,
+                      drift = 0, reverse = FALSE) {
   # Recycle drifts if needed
   if (length(drift) != len) {
     drift <- rep(drift, len)[seq_len(len)]
@@ -96,19 +107,31 @@ apply.lrr <- function(F, lrr, len = 1, only.new = FALSE, drift = 0) {
     stop("Wrong length of LRR")
 
   # Run the actual LRR
-  F <- c(F, rep(NA, len))
-  for (i in 1:len)
-    F[N+i] <- sum(F[(N+i-r) : (N+i-1)]*lrr) + drift[i]
+  if (!reverse) {
+    F <- c(F, rep(NA, len))
+    for (i in 1:len)
+      F[N+i] <- sum(F[(N+i-r) : (N+i-1)]*lrr) + drift[i]
 
-  if (only.new) F[(N+1):(N+len)] else F
+    if (only.new) F[(N+1):(N+len)] else F
+  } else {
+    F <- c(rep(NA, len), F)
+
+    for (i in 1:len)
+      F[len-i+1] <- sum(F[(len-i+1 + 1) : (len-i+1 + r)]*lrr) + drift[len-i+1]
+
+    if (only.new) F[1:len] else F
+  }
 }
 
-
 rforecast.1d.ssa <- function(x, groups, len = 1,
                              base = c("reconstructed", "original"),
                              only.new = TRUE,
+                             reverse = FALSE,
                              ...,
                              drop = TRUE, drop.attributes = FALSE, cache = TRUE) {
+  if (is.shaped(x))
+    stop("`forecasting is not implemented for shaped SSA case yet")
+
   if (x$circular)
     stop("forecasting is not properly defined for circular SSA")
 
@@ -123,18 +146,19 @@ rforecast.1d.ssa <- function(x, groups, len = 1,
     r <- reconstruct(x, groups = groups, ..., cache = cache)
 
   # Calculate the LRR corresponding to groups
-  lf <- lrr(x, groups = groups, drop = FALSE)
+  lf <- lrr(x, groups = groups, reverse = reverse, drop = FALSE)
   stopifnot(length(lf) == length(groups))
 
   out <- list()
   for (i in seq_along(groups)) {
     group <- groups[[i]]
 
+    F <- if (identical(base, "reconstructed")) as.vector(r[[i]]) else .F(x)
+
     # Calculate the forecasted values
-    out[[i]] <- apply.lrr(if (identical(base, "reconstructed")) r[[i]] else .F(x),
-                          lf[[i]], len, only.new = only.new)
+    out[[i]] <- apply.lrr(F, lf[[i]], len, only.new = only.new, reverse = reverse)
     out[[i]] <- .apply.attributes(x, out[[i]],
-                                  fixup = TRUE,
+                                  fixup = TRUE, reverse = reverse,
                                   only.new = only.new, drop = drop.attributes)
   }
 
@@ -251,6 +275,9 @@ vforecast.1d.ssa <- function(x, groups, len = 1,
                              only.new = TRUE,
                              ...,
                              drop = TRUE, drop.attributes = FALSE) {
+  if (is.shaped(x))
+    stop("`forecasting is not implemented for shaped SSA case yet")
+
   if (x$circular)
     stop("forecasting is not properly defined for circular SSA")
 
@@ -270,7 +297,7 @@ vforecast.1d.ssa <- function(x, groups, len = 1,
   V <- if (nv(x) >= desired) .V(x) else NULL
 
   # Grab the FFT plan
-  fft.plan <- fft.plan.1d(N)
+  fft.plan <- fft.plan.1d(N, L = L)
 
   out <- list()
   for (i in seq_along(groups)) {
@@ -326,7 +353,9 @@ vforecast.mssa <- function(x, groups, len = 1,
   N <- N.res + switch(direction, column = L, row = K) - 1
 
   # Grab the FFT plan
-  fft.plan <- lapply(N, fft.plan.1d)
+  fft.plan <- switch(direction,
+                     column = lapply(N, fft.plan.1d, L = L),
+                     row = mapply(fft.plan.1d, N = N, L = L + len + K - 1))
 
   cK <- cumsum(K)
   cKs <- cK - K + 1
